import assert from 'assert';
import { THREAD_COUNT, CURSOR_BUFFER_SIZE, THREAD_COUNTER_BYTE_LENGTH } from './constants';

const checkThreadCounterHexString = threadCounter => {
  assert.strictEqual(threadCounter.length, 12, 'threadCounter in hex form must have length equal to 12');
};

export const initThreadArray = () => {
  const threadCounters = new Array(THREAD_COUNT);
  threadCounters.fill(0);
  return threadCounters;
};
export const threadArrayToCursor = threadArray => {
  assert.strictEqual(threadArray.length, THREAD_COUNT, 'Cursor must be represented by array of 256 numbers');
  const cursorBuffer = Buffer.alloc(CURSOR_BUFFER_SIZE);

  for (let i = 0; i < threadArray.length; ++i) {
    const threadCounter = threadArray[i];
    const threadCounterBuffer = threadCounterToBuffer(threadCounter);
    threadCounterBuffer.copy(cursorBuffer, i * THREAD_COUNTER_BYTE_LENGTH);
  }

  return cursorBuffer.toString('base64');
};
export const cursorToThreadArray = cursor => {
  if (cursor == null) return initThreadArray();
  const cursorBuffer = Buffer.from(cursor, 'base64');
  assert.strictEqual(cursorBuffer.length, CURSOR_BUFFER_SIZE, 'Wrong size of cursor buffer');
  const threadCounters = new Array(THREAD_COUNT);

  for (let i = 0; i < cursorBuffer.length / THREAD_COUNTER_BYTE_LENGTH; i++) {
    threadCounters[i] = hexStringToThreadCounter(cursorBuffer.slice(i * 6, (i + 1) * 6).toString('hex'));
  }

  return threadCounters;
};
export const threadCounterToHexString = threadCounter => threadCounter.toString(16).padStart(12, '0');
export const hexStringToThreadCounter = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const num = parseInt(threadCounter, 16);
  assert(!Number.isNaN(num));
  return num;
};
export const threadCounterHexStringToBuffer = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const b = Buffer.alloc(THREAD_COUNTER_BYTE_LENGTH);

  for (let hexIndex = 0, hexPairIndex = 0; hexIndex < threadCounter.length; hexIndex += 2, hexPairIndex++) {
    b[hexPairIndex] = Buffer.from(threadCounter.substring(hexIndex, hexIndex + 2), 'hex')[0];
  }

  return b;
};
export const threadCounterToBuffer = threadCounter => {
  return threadCounterHexStringToBuffer(threadCounterToHexString(threadCounter));
};
//# sourceMappingURL=cursor-operations.js.map