"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.threadCounterToBuffer = exports.threadCounterHexStringToBuffer = exports.hexStringToThreadCounter = exports.threadCounterToHexString = exports.cursorToThreadArray = exports.threadArrayToCursor = exports.initThreadArray = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const checkThreadCounterHexString = threadCounter => {
  _assert.default.strictEqual(threadCounter.length, 12, 'threadCounter in hex form must have length equal to 12');
};

const initThreadArray = () => {
  const threadCounters = new Array(_constants.THREAD_COUNT);
  threadCounters.fill(0);
  return threadCounters;
};

exports.initThreadArray = initThreadArray;

const threadArrayToCursor = threadArray => {
  _assert.default.strictEqual(threadArray.length, _constants.THREAD_COUNT, 'Cursor must be represented by array of 256 numbers');

  const cursorBuffer = Buffer.alloc(_constants.CURSOR_BUFFER_SIZE);

  for (let i = 0; i < threadArray.length; ++i) {
    const threadCounter = threadArray[i];
    const threadCounterBuffer = threadCounterToBuffer(threadCounter);
    threadCounterBuffer.copy(cursorBuffer, i * _constants.THREAD_COUNTER_BYTE_LENGTH);
  }

  return cursorBuffer.toString('base64');
};

exports.threadArrayToCursor = threadArrayToCursor;

const cursorToThreadArray = cursor => {
  if (cursor == null) return initThreadArray();
  const cursorBuffer = Buffer.from(cursor, 'base64');

  _assert.default.strictEqual(cursorBuffer.length, _constants.CURSOR_BUFFER_SIZE, 'Wrong size of cursor buffer');

  const threadCounters = new Array(_constants.THREAD_COUNT);

  for (let i = 0; i < cursorBuffer.length / _constants.THREAD_COUNTER_BYTE_LENGTH; i++) {
    threadCounters[i] = hexStringToThreadCounter(cursorBuffer.slice(i * 6, (i + 1) * 6).toString('hex'));
  }

  return threadCounters;
};

exports.cursorToThreadArray = cursorToThreadArray;

const threadCounterToHexString = threadCounter => threadCounter.toString(16).padStart(12, '0');

exports.threadCounterToHexString = threadCounterToHexString;

const hexStringToThreadCounter = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const num = parseInt(threadCounter, 16);
  (0, _assert.default)(!Number.isNaN(num));
  return num;
};

exports.hexStringToThreadCounter = hexStringToThreadCounter;

const threadCounterHexStringToBuffer = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const b = Buffer.alloc(_constants.THREAD_COUNTER_BYTE_LENGTH);

  for (let hexIndex = 0, hexPairIndex = 0; hexIndex < threadCounter.length; hexIndex += 2, hexPairIndex++) {
    b[hexPairIndex] = Buffer.from(threadCounter.substring(hexIndex, hexIndex + 2), 'hex')[0];
  }

  return b;
};

exports.threadCounterHexStringToBuffer = threadCounterHexStringToBuffer;

const threadCounterToBuffer = threadCounter => {
  return threadCounterHexStringToBuffer(threadCounterToHexString(threadCounter));
};

exports.threadCounterToBuffer = threadCounterToBuffer;
//# sourceMappingURL=cursor-operations.js.map